@@ -304,7 +304,8 @@ def _checkClassAttribute(attr, c, code) :
 def _checkModuleAttribute(attr, module, code, ref) :
     refModule = module.modules.get(ref)
     if refModule and refModule.attributes != None :
-        if attr not in refModule.attributes :
+        if attr not in refModule.attributes and \
+           not utils.endswith(ref, '.' + attr) :
             code.addWarning(msgs.INVALID_MODULE_ATTR % attr)
 
     refClass = module.classes.get(ref)
@@ -326,12 +327,14 @@ def _getGlobalName(name, func) :
     return name
 
 
-def _makeConstant(stack, index, factoryFunction) :
+def _makeConstant(code, index, factoryFunction) :
     "Build a constant on the stack ((), [], or {})"
     if index > 0 :
-        stack[-index:] = [ factoryFunction(stack[-index:]) ]
+        code.stack[-index:] = [ factoryFunction(code.stack[-index:]) ]
+        if OP.POP_TOP(code.nextOpInfo()[0]) :
+            code.addWarning(msgs.POSSIBLE_STMT_WITH_NO_EFFECT)
     else :
-        stack.append(factoryFunction())
+        code.pushStack(factoryFunction())
 
 
 def _hasGlobal(operand, module, func, main) :
@@ -406,7 +409,7 @@ def _handleImport(code, operand, module, main, fromName) :
 
 def _handleImportFrom(code, operand, module, main) :
     fromName = code.stack[-1].data
-    code.stack.append(Stack.Item(operand, types.ModuleType))
+    code.pushStack(Stack.Item(operand, types.ModuleType))
     _handleImport(code, operand, module, main, fromName)
 
 
@@ -476,7 +479,8 @@ def _getConstant(code, module, data) :
     return None
 
 _UNCHECKABLE_FORMAT_STACK_TYPES = \
-      (Stack.TYPE_UNKNOWN, Stack.TYPE_FUNC_RETURN, Stack.TYPE_ATTRIBUTE,)
+      (Stack.TYPE_UNKNOWN, Stack.TYPE_FUNC_RETURN, Stack.TYPE_ATTRIBUTE,
+       Stack.TYPE_GLOBAL,)
 
 def _getFormatString(code, codeSource) :
     if len(code.stack) <= 1 :
@@ -634,6 +638,11 @@ class Code :
                     return op
         raise RuntimeError('Could not find first opcode in function')
 
+    def pushStack(self, item) :
+        self.stack.append(item)
+        if OP.POP_TOP(self.nextOpInfo()[0]) :
+            self.addWarning(msgs.POSSIBLE_STMT_WITH_NO_EFFECT)
+
     def popStack(self) :
         if self.stack :
             del self.stack[-1]
@@ -738,7 +747,7 @@ class CodeSource :
         self.main = main
         self.in_class = in_class
         self.code = code
-        self.calling_code = None
+        self.calling_code = []
 
 def _checkException(code, name) :
     if code.stack and code.stack[-1].type == Stack.TYPE_EXCEPT :
@@ -782,11 +791,11 @@ def _checkLoadGlobal(codeSource, code, varname) :
     should_check = 1
     if code.func_code.co_name == utils.LAMBDA :
         # this could really be a local reference, check first
-        if not codeSource.main and \
-           hasattr(codeSource.calling_code, 'function') and \
-           varname in codeSource.calling_code.function.func_code.co_varnames :
-            _handleLoadLocal(code, codeSource, varname)
-            should_check = 0
+        if not codeSource.main and codeSource.calling_code:
+            func = getattr(codeSource.calling_code[-1], 'function', None)
+            if func is not None and varname in func.func_code.co_varnames :
+                _handleLoadLocal(code, codeSource, varname)
+                should_check = 0
 
     if should_check :
         # make sure we remember each global ref to check for unused
@@ -797,8 +806,6 @@ def _checkLoadGlobal(codeSource, code, varname) :
 
 def _LOAD_NAME(oparg, operand, codeSource, code) :
     _checkLoadGlobal(codeSource, code, operand)
-    if OP.POP_TOP(code.nextOpInfo()[0]) :
-        code.addWarning(msgs.POSSIBLE_STMT_WITH_NO_EFFECT)
 
     # if there was from XXX import *, _* names aren't imported
     if codeSource.module.modules.has_key(operand) and \
@@ -810,17 +817,16 @@ def _LOAD_NAME(oparg, operand, codeSource, code) :
         opType, const = types.NoneType, 0
     elif operand == 'Ellipsis' :
         opType, const = types.EllipsisType, 1
-    code.stack.append(Stack.Item(operand, opType, const))
+    code.pushStack(Stack.Item(operand, opType, const))
 
 _LOAD_GLOBAL = _LOAD_NAME
 
 def _LOAD_DEREF(oparg, operand, codeSource, code) :
     if type(oparg) == types.IntType :
+        argname = code.func_code.co_varnames[oparg]
+        code.pushStack(Stack.Item(argname, types.StringType))
         if code.func_code.co_name != utils.LAMBDA :
-            argused = code.func_code.co_varnames[oparg]
-            code.unusedLocals[argused] = None
-        if OP.POP_TOP(code.nextOpInfo()[0]) :
-            code.addWarning(msgs.POSSIBLE_STMT_WITH_NO_EFFECT)
+            code.unusedLocals[argname] = None
     else :
         _LOAD_GLOBAL(oparg, operand, codeSource, code)
 
@@ -832,7 +838,7 @@ def _DELETE_NAME(oparg, operand, codeSource, code) :
 _DELETE_GLOBAL = _DELETE_NAME
 
 def _LOAD_CONST(oparg, operand, codeSource, code) :
-    code.stack.append(Stack.Item(operand, type(operand), 1))
+    code.pushStack(Stack.Item(operand, type(operand), 1))
     if type(operand) == types.CodeType :
         name = operand.co_name
         obj = code.codeObjects.get(name, None)
@@ -843,8 +849,6 @@ def _LOAD_CONST(oparg, operand, codeSource, code) :
             code.codeObjects[name] = operand
         elif cfg().redefiningFunction :
             code.addWarning(msgs.REDEFINING_ATTR % (name, obj.co_firstlineno))
-    elif OP.POP_TOP(code.nextOpInfo()[0]) :
-        code.addWarning(msgs.POSSIBLE_STMT_WITH_NO_EFFECT)
 
 
 def _checkLocalShadow(code, module, varname) :
@@ -869,10 +873,8 @@ def _handleLoadLocal(code, codeSource, varname) :
                     msgs.LOCAL_DELETED, msgs.VAR_USED_BEFORE_SET)
 
 def _LOAD_FAST(oparg, operand, codeSource, code) :
-    code.stack.append(Stack.Item(operand, type(operand)))
+    code.pushStack(Stack.Item(operand, type(operand)))
     _handleLoadLocal(code, codeSource, operand)
-    if OP.POP_TOP(code.nextOpInfo()[0]) :
-        code.addWarning(msgs.POSSIBLE_STMT_WITH_NO_EFFECT)
 
 def _STORE_FAST(oparg, operand, codeSource, code) :
     if not code.updateCheckerArgs(operand) :
@@ -960,12 +962,9 @@ def _COMPARE_OP(oparg, operand, codeSource, code) :
     _handleComparison(code.stack, operand)
 
 def _IMPORT_NAME(oparg, operand, codeSource, code) :
-    code.stack.append(Stack.Item(operand, types.ModuleType))
+    code.pushStack(Stack.Item(operand, types.ModuleType))
     nextOp = code.nextOpInfo()[0]
     if not OP.IMPORT_FROM(nextOp) and not OP.IMPORT_STAR(nextOp) :
-        # handle import xml.sax as sax
-        if OP.LOAD_ATTR(nextOp) :
-            operand = code.popNextOp()[2]
         _handleImport(code, operand, codeSource.module, codeSource.main, None)
 
 def _IMPORT_FROM(oparg, operand, codeSource, code) :
@@ -983,7 +982,7 @@ def _IMPORT_STAR(oparg, operand, codeSource, code) :
 
 def _DUP_TOP(oparg, operand, codeSource, code) :
     if len(code.stack) > 0 :
-        code.stack.append(code.stack[-1])
+        code.pushStack(code.stack[-1])
 
 def _pop2(oparg, operand, codeSource, code) :
     if len(code.stack) >= 2 :
@@ -1009,15 +1008,15 @@ def _CALL_FUNCTION_VAR_KW(oparg, operand, codeSource, code) :
 def _MAKE_FUNCTION(oparg, operand, codeSource, code) :
     newValue = Stack.makeFuncReturnValue(code.stack[-1], oparg)
     code.popStackItems(oparg+1)
-    code.stack.append(newValue)
+    code.pushStack(newValue)
 _MAKE_CLOSURE = _MAKE_FUNCTION
 
 def _BUILD_MAP(oparg, operand, codeSource, code) :
-    _makeConstant(code.stack, oparg, Stack.makeDict)
+    _makeConstant(code, oparg, Stack.makeDict)
 def _BUILD_TUPLE(oparg, operand, codeSource, code) :
-    _makeConstant(code.stack, oparg, Stack.makeTuple)
+    _makeConstant(code, oparg, Stack.makeTuple)
 def _BUILD_LIST(oparg, operand, codeSource, code) :
-    _makeConstant(code.stack, oparg, Stack.makeList)
+    _makeConstant(code, oparg, Stack.makeList)
 
 def _UNARY_CONVERT(oparg, operand, codeSource, code) :
     if len(code.stack) > 0 :
@@ -1047,7 +1046,7 @@ def _UNARY_INVERT(oparg, operand, codeSource, code) :
 
 def _popStackRef(code, operand, count = 2) :
     code.popStackItems(count)
-    code.stack.append(Stack.Item(operand, Stack.TYPE_UNKNOWN))
+    code.pushStack(Stack.Item(operand, Stack.TYPE_UNKNOWN))
 
 def _pop(oparg, operand, codeSource, code) :
     code.popStack()
@@ -1058,6 +1057,9 @@ _NUMERIC_TYPES = (types.IntType, types.FloatType)
 
 # FIXME: This is pathetically weak, need to handle more types
 def _coerce_type(code) :
+    if OP.POP_TOP(code.nextOpInfo()[0]) :
+        code.addWarning(msgs.POSSIBLE_STMT_WITH_NO_EFFECT)
+
     if len(code.stack) >= 2 :
         s1, s2 = code.stack[-2:]
         s1type = s1.getType(code.typeMap)
@@ -1072,7 +1074,7 @@ def _BINARY_ADD(oparg, operand, codeSource, code) :
                             stack[-1].type == stack[-2].type) :
         value = stack[-2].data + stack[-1].data
         code.popStackItems(2)
-        code.stack.append(Stack.Item(value, type(value), 1))
+        code.pushStack(Stack.Item(value, type(value), 1))
     else :
         _coerce_type(code)
         code.popStack()
@@ -1086,6 +1088,9 @@ def _BINARY_SUBTRACT(oparg, operand, codeSource, code) :
 _BINARY_POWER = _BINARY_SUBTRACT
 
 def _BINARY_SUBSCR(oparg, operand, codeSource, code) :
+    if OP.POP_TOP(code.nextOpInfo()[0]) :
+        code.addWarning(msgs.POSSIBLE_STMT_WITH_NO_EFFECT)
+
     if len(code.stack) >= 2 :
         stack = code.stack
         varType = code.typeMap.get(str(stack[-2].data), [])
@@ -1102,6 +1107,9 @@ def _isint(stackItem, code) :
     return types.IntType in stackTypes
 
 def _BINARY_DIVIDE(oparg, operand, codeSource, code) :
+    if OP.POP_TOP(code.nextOpInfo()[0]) :
+        code.addWarning(msgs.POSSIBLE_STMT_WITH_NO_EFFECT)
+
     if cfg().intDivide and len(code.stack) >= 2 :
         if _isint(code.stack[-1], code) and _isint(code.stack[-2], code) :
             code.addWarning(msgs.INTEGER_DIVISION % tuple(code.stack[-2:]))
@@ -1119,6 +1127,9 @@ def _BINARY_MULTIPLY(oparg, operand, codeSource, code) :
     code.popStack()
 
 def _BINARY_MODULO(oparg, operand, codeSource, code) :
+    if OP.POP_TOP(code.nextOpInfo()[0]) :
+        code.addWarning(msgs.POSSIBLE_STMT_WITH_NO_EFFECT)
+
     _getFormatWarnings(code, codeSource)
     code.popStack()
 
@@ -1127,8 +1138,8 @@ def _ROT_TWO(oparg, operand, codeSource, code) :
         del code.stack[-2]
 
 def _SETUP_EXCEPT(oparg, operand, codeSource, code) :
-    code.stack.append(Stack.Item(None, Stack.TYPE_EXCEPT))
-    code.stack.append(Stack.Item(None, Stack.TYPE_EXCEPT))
+    code.pushStack(Stack.Item(None, Stack.TYPE_EXCEPT))
+    code.pushStack(Stack.Item(None, Stack.TYPE_EXCEPT))
 
 def _LINE_NUM(oparg, operand, codeSource, code) :
     code.lastLineNum = oparg
@@ -1158,29 +1169,55 @@ def _jump(oparg, operand, codeSource, code) :
                 code.addWarning(msgs.USING_METHOD_AS_ATTR % name)
 _JUMP_ABSOLUTE = _jump
 
-def _jump_conditional(oparg, operand, codeSource, code) :
-    if code.stack :
-        if code.stack[-1].const and cfg().constantConditions and \
-           (code.stack[-1].data != 1 or cfg().constant1) :
-            code.addWarning(msgs.CONSTANT_CONDITION % str(code.stack[-1]))
+def _is_unreachable(code, topOfStack, branch, if_false) :
+    # Are we are checking exceptions, but we not catching all exceptions?
+    if (topOfStack.type == Stack.TYPE_COMPARISON and 
+        topOfStack.data[1] == 'exception match' and 
+        topOfStack.data[2] is not Exception) :
+        return 1
 
-        # Remove branches to code when we are checking exceptions
-        # and we aren't catching all exceptions
+    # do we possibly have while 1: ?
+    if not (topOfStack.const and topOfStack.data == 1 and if_false) :
+        return 0
+
+    # get the op just before the branch (ie, -3)
+    op, oparg, i, extended_arg = OP.getInfo(code.bytes, branch - 3, 0)
+    # are we are jumping to before the while 1: (LOAD_CONST, JUMP_IF_FALSE)
+    if not (OP.JUMP_ABSOLUTE(op) and oparg == (code.index - 3*3)) :
+        return 0
+
+    # check if we break out of the loop
+    i = code.index
+    while i < branch :
+        op, oparg, i, extended_arg = OP.getInfo(code.bytes, i, extended_arg)
+        if OP.BREAK_LOOP(op) :
+            return 0
+    return 1
+
+def _jump_conditional(oparg, operand, codeSource, code, if_false) :
+    if code.stack :
         topOfStack = code.stack[-1]
-        if topOfStack.type == Stack.TYPE_COMPARISON and \
-           topOfStack.data[1] == 'exception match' and \
-           topOfStack.data[2] is not Exception :
+        if topOfStack.const and cfg().constantConditions and \
+           (topOfStack.data != 1 or cfg().constant1) :
+            code.addWarning(msgs.CONSTANT_CONDITION % str(topOfStack))
+
+        if _is_unreachable(code, topOfStack, code.index + oparg, if_false) :
             code.removeBranch(code.index + oparg)
                 
     _jump(oparg, operand, codeSource, code)
-_JUMP_IF_FALSE = _JUMP_IF_TRUE = _jump_conditional
+
+def _JUMP_IF_FALSE(oparg, operand, codeSource, code) :
+    _jump_conditional(oparg, operand, codeSource, code, 1)
+
+def _JUMP_IF_TRUE(oparg, operand, codeSource, code) :
+    _jump_conditional(oparg, operand, codeSource, code, 0)
 
 def _JUMP_FORWARD(oparg, operand, codeSource, code) :
     _jump(oparg, operand, codeSource, code)
     code.remove_unreachable_code(code.label)
 
 def _RETURN_VALUE(oparg, operand, codeSource, code) :
-    if codeSource.calling_code is None :
+    if not codeSource.calling_code :
         code.addReturn()
 
 def _EXEC_STMT(oparg, operand, codeSource, code) :
