@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2001, MetaSlash Inc.  All rights reserved.
+# Copyright (c) 2001-2002, MetaSlash Inc.  All rights reserved.
 
 """
 Object to hold information about functions.
@@ -11,16 +11,27 @@ import string
 
 _ARGS_ARGS_FLAG = 4
 _KW_ARGS_FLAG = 8
+_CO_FLAGS_MASK = _ARGS_ARGS_FLAG + _KW_ARGS_FLAG
 
+class FakeCode :
+    "This is a holder class for code objects (so we can modify them)"
+    def __init__(self, code, varnames = None) :
+        for attr in dir(code):
+            try:
+                setattr(self, attr, getattr(code, attr))
+            except:
+                pass
+        if varnames is not None:
+            self.co_varnames = varnames
 
 class FakeFunction :
     "This is a holder class for turning code at module level into a function"
 
-    def __init__(self, name, code, func_globals = {}) :
+    def __init__(self, name, code, func_globals = {}, varnames = None) :
         self.func_name = self.__name__ = name
         self.func_doc  = self.__doc__  = "ignore"
 
-        self.func_code = code
+        self.func_code = FakeCode(code, varnames)
         self.func_defaults = None
         self.func_globals = func_globals
 
@@ -63,8 +74,8 @@ class Function :
         func_code = self.function.func_code
         return func_code.co_varnames[func_code.co_argcount]
 
-def create_fake(name, code, func_globals = {}) :
-    return Function(FakeFunction(name, code, func_globals))
+def create_fake(name, code, func_globals = {}, varnames = None) :
+    return Function(FakeFunction(name, code, func_globals, varnames))
 
 def create_from_file(file, filename, module) :
     # Make sure the file is at the beginning
@@ -80,6 +91,9 @@ def create_from_file(file, filename, module) :
     code = compile(codestr, filename, 'exec')
     return Function(FakeFunction('__main__', code, module.__dict__))
 
+def _co_flags_equal(o1, o2) :
+    return (o1.co_flags & _CO_FLAGS_MASK) == (o2.co_flags & _CO_FLAGS_MASK)
+    
 def same_signature(func, object) :
     '''Return a boolean value if the <func> has the same signature as
        a function with the same name in <object> (ie, an overriden method)'''
@@ -90,6 +104,6 @@ def same_signature(func, object) :
     except AttributeError :
         return 1
 
-    return base_func_code.co_flags == func.func_code.co_flags and \
+    return _co_flags_equal(base_func_code, func.func_code) and \
            base_func_code.co_argcount == func.func_code.co_argcount
 
