@@ -68,7 +68,7 @@ IMPLICIT_AND_EXPLICIT_RETURNS = "Function returns a value and also implicitly re
 INCONSISTENT_RETURN_TYPE = "Function return types are inconsistent"
 CODE_UNREACHABLE = "Code appears to be unreachable"
 CONSTANT_CONDITION = "Using a conditional statement with a constant value (%s)"
-DONT_RETURN_NONE = "%s should never return None, raise an exception if not found"
+DONT_RETURN_NONE = "%s should not return None, raise an exception if not found"
 
 INVALID_FORMAT = "Invalid format string, problem starts near: '%s'"
 INVALID_FORMAT_COUNT = "Format string argument count (%d) doesn't match arguments (%d)"
@@ -94,6 +94,7 @@ MODIFYING_DEFAULT_ARG = "Modifying parameter (%s) with a default value may have
 USING_SELF_IN_REPR = "Using `self` in __repr__ method"
 USING_NONE_RETURN_VALUE = "Using the return value from (%s) which is always None"
 
+USING_PROPERTIES_IN_CLASSIC_CLASS = "Using property (%s) in classic class %s may not work"
 USING_SLOTS_IN_CLASSIC_CLASS = "Using __slots__ in classic class %s is not used by Python"
 EMPTY_SLOTS = "__slots__ are empty in %s"
 
