@@ -47,12 +47,20 @@ def _checkNoSelfArg(func, warnings) :
         warnings.append(Warning(code, code, msgs.SELF_IS_ARG))
 
 
+def _checkSubclass(c1, c2):
+    try:
+        return issubclass(c1.classObject, c2.classObject)
+    except (TypeError, AttributeError):
+        return 0
+
+
 _IGNORE_RETURN_TYPES = ( Stack.TYPE_FUNC_RETURN, Stack.TYPE_ATTRIBUTE,
                          Stack.TYPE_GLOBAL, Stack.TYPE_COMPARISON,
                          Stack.TYPE_UNKNOWN)
 
 def _checkReturnWarnings(code) :
-    if code.func_code.co_name in ('__getattr__', '__getattribute__') :
+    is_getattr = code.func_code.co_name in ('__getattr__', '__getattribute__')
+    if is_getattr :
         for line, retval, dummy in code.returnValues :
             if retval.isNone() :
                 err = msgs.DONT_RETURN_NONE % code.func_code.co_name
@@ -72,6 +80,10 @@ def _checkReturnWarnings(code) :
                                 lastReturn[0]+1)
                 break
 
+    # getattr can return different types, so don't warn about inconsistency
+    if is_getattr :
+        return
+
     returnType, returnData = None, None
     for line, value, dummy in code.returnValues :
         if not value.isNone() :
@@ -79,6 +91,7 @@ def _checkReturnWarnings(code) :
             if returnType is None and valueType not in _IGNORE_RETURN_TYPES :
                 returnData = value
                 returnType = valueType
+                continue
 
             # always ignore None, None can be returned w/any other type
             # FIXME: if we stored func return values, we could do better
@@ -86,13 +99,17 @@ def _checkReturnWarnings(code) :
                valueType not in _IGNORE_RETURN_TYPES and \
                returnData.type not in _IGNORE_RETURN_TYPES :
                 ok = returnType in (type(value.data), valueType)
-                if ok and returnType == types.TupleType :
-                    # FIXME: this isn't perfect, if len == 0
-                    # the length can really be 0 OR unknown
-                    # we shouldn't check the lengths for equality
-                    # ONLY IF one of the lengths is truly unknown
-                    if returnData.length > 0 and value.length > 0:
-                        ok = returnData.length == value.length
+                if ok :
+                    if returnType == types.TupleType :
+                        # FIXME: this isn't perfect, if len == 0
+                        # the length can really be 0 OR unknown
+                        # we shouldn't check the lengths for equality
+                        # ONLY IF one of the lengths is truly unknown
+                        if returnData.length > 0 and value.length > 0:
+                            ok = returnData.length == value.length
+                else :
+                    ok = _checkSubclass(returnType, valueType) or \
+                         _checkSubclass(valueType, returnType)
                 if not ok :
                     code.addWarning(msgs.INCONSISTENT_RETURN_TYPE, line)
 
@@ -124,17 +141,16 @@ def _checkUnusedParam(var, line, func, code) :
             (cfg().varArgumentsUsed or func.varArgName() != var)) :
             code.addWarning(msgs.UNUSED_PARAMETER % var, code.func_code)
 
-def _handleLambda(func_code, code, codeSource):
+def _handleNestedCode(func_code, code, codeSource):
     nested = not (codeSource.main or codeSource.in_class)
     if func_code.co_name == utils.LAMBDA or nested:
-        utils.debug(' handling lambda')
+        utils.debug(' handling nested code')
         varnames = None
         if nested and func_code.co_name != utils.LAMBDA:
-            varnames = codeSource.calling_code.function.func_code.co_varnames 
+            varnames = func_code.co_varnames + \
+                     codeSource.calling_code[-1].function.func_code.co_varnames
         code.init(function.create_fake(func_code.co_name, func_code, {},
                                        varnames))
-                      
-        # I sure hope there can't be/aren't lambda's within lambda's
         _checkCode(code, codeSource)
 
 def _findUnreachableCode(code) :
@@ -180,11 +196,10 @@ def _checkFunction(module, func, c = None, main = 0, in_class = 0) :
         _checkCode(code, codeSource)
 
         # handle lambdas
-        old_callling_code = codeSource.calling_code
-        codeSource.calling_code = func
+        codeSource.calling_code.append(func)
         for func_code in code.codeObjects.values() :
-            _handleLambda(func_code, code, codeSource)
-        codeSource.calling_code = old_callling_code
+            _handleNestedCode(func_code, code, codeSource)
+        del codeSource.calling_code[-1]
 
         if not in_class :
             _findUnreachableCode(code)
@@ -266,7 +281,11 @@ def _baseInitCalled(classInitInfo, base, functionsCalled) :
 
     # ok, do this the hard way, there may be aliases, so check here
     names = string.split(initName, '.')
-    obj = sys.modules[names[0]]
+    try:
+        # i think this can raise an exception if the module is a library (.so)
+        obj = sys.modules[names[0]]
+    except KeyError:
+        return 1
     for i in range(1, len(names)) :
         obj = getattr(obj, names[i], None)
         if obj is None:
@@ -337,10 +356,10 @@ def getBlackList(moduleList) :
 
 def getStandardLibrary() :
     if cfg().ignoreStandardLibrary :
-        import os.path
-        from distutils import sysconfig
-
         try :
+            import os.path
+            from distutils import sysconfig
+
             std_lib = sysconfig.get_python_lib()
             path = os.path.split(std_lib)
             if path[1] == 'site-packages' :
@@ -419,6 +438,12 @@ try:
 except NameError:
     object = _SuppressionError
 
+# Create property for pre-2.2 interpreters
+try :
+    if property: pass
+except NameError:
+    property = None
+
 def _findClassWarnings(module, c, class_code,
                        globalRefs, warnings, suppressions) :
     classSuppress = getSuppression(str(c.classObject), suppressions, warnings)
@@ -473,9 +498,10 @@ def _findClassWarnings(module, c, class_code,
         err = msgs.UNUSED_MEMBERS % (string.join(memberList, ', '), c.name)
         warnings.append(Warning(filename, c.getFirstLine(), err))
 
+    newStyleClass = issubclass(c.classObject, object)
+
     slots = c.statics.get('__slots__')
     if slots is not None and cfg().slots:
-        newStyleClass = issubclass(c.classObject, object)
         lineNum = c.lineNums['__slots__']
         if not newStyleClass:
             err = msgs.USING_SLOTS_IN_CLASSIC_CLASS % c.name
@@ -484,6 +510,13 @@ def _findClassWarnings(module, c, class_code,
             err = msgs.EMPTY_SLOTS % c.name
             warnings.append(Warning(filename, lineNum, err))
 
+    if not newStyleClass and property is not None and \
+       cfg().classicProperties:
+        for static in c.statics.keys():
+            if type(getattr(c.classObject, static)) == property:
+                err = msgs.USING_PROPERTIES_IN_CLASSIC_CLASS % (static, c.name)
+                warnings.append(Warning(filename, c.lineNums[static], err))
+
     if cfg().noDocClass and c.classObject.__doc__ == None :
         method = c.methods.get(utils.INIT, None)
         if method != None :
@@ -546,6 +579,10 @@ def find(moduleList, initialCfg, suppressions = None) :
                                        msgs.VAR_NOT_USED, prefix))
         if cfg().importUsed :
             if module.moduleName != utils.INIT or cfg().packageImportUsed :
+                # always ignore readline module, if [raw_]input() is used
+                if globalRefs.has_key('input') or \
+                   globalRefs.has_key('raw_input'):
+                    globalRefs['readline'] = 0
                 warnings.extend(_getUnused(module, globalRefs, module.modules,
                                            msgs.IMPORT_NOT_USED))
 
